# -----------------------------------------------------------------------------
# (C) British Crown Copyright 2017-2021 Met Office.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 
#
# Plot Fig 6 from "An inter-comparison of the mass budget of the Arctic sea ice in 
# CMIP6 models", Keen et al (2021).
# XX denotes local pathnames/files etc required 

import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import read_budgetfiles as rb
import budget_utils as bu
from SIMIP_Massbudget_DataDic import DataDic # XX Dictionary defining all the filenames etc
from SIMIP_Massbudget_BudgetLineDic import BudgetLineDic # XX Dictionary defining budget line colours etc
import sys 
 
# Input data path
#-----------------
path = ' ' # XX

# All CMIP6 models - ordered by sea ice component
ModelNames_CMIP6_neworder = ['NCAR_CESM2_CAM',   # CICE
          'NCAR_CESM2_WACCM', 		# CICE
          'NCC_NorESM2-LM',       	# CICE
          'NCC_NorESM2-MM',       	# CICE
          'MetO_UKESM1',      		# CICE (GSI8.1)
          'MetO_GC3.1_ORCA1',  		# CICE (GSI8.1)    
          'MetO_GC3.1_ORCA025',  	# CICE (GSI8.1)
          'CSIRO_ACCESS-CM2', 		# CICE (GSI8.1)
          'UCLouvain_EC-Earth3-Veg',    # LIM3
          'UCLouvain_EC-Earth3',        # LIM3
          'IPSL_CM6A-LR',          	# LIM3
          'CCCma_CanESM5_x1',          	# LIM2
          'GFDL-CM4',         		# SIS2
          'GFDL-ESM4',        		# SIS2
          'MRI_ESM2.0']         	# MRI

ModelNames = ModelNames_CMIP6_neworder
plotTag = 'CMIP6_models'

nskip = 4
seconds_in_month = 60*60*24*30

plotdir='Plot_files/'

#-----------------------------------------------

nmodels = len(ModelNames)
Var_labels = BudgetLineDic['ice']['Var_labels']
#nvars = len(Var_labels)
Var_labels_sub=Var_labels[0:4]+[Var_labels[7]]
var_indices_sub=[0,1,2,3,7]
nvars = len(Var_labels_sub)

plot_titles=['(a) Basal growth','(b) Frazil','(c) Top melt','(d) Basal melt','(e) Dynamics']

# Data processing ---------------------------------------------------------------
# Make the annual-mean ensemble-mean for each model, plus also the multi-model mean

# Plotting -----------------------------------------------------------------------------
matplotlib.rcParams.update({'font.size':8})

# define figure size
figSize = (8.3,11.7) # A4 portrait
#figSize = (11.7,8.3) # A4 landscape

# define plot geometry
geomX = 2
geomY = 3
maxPanels = geomX * geomY

# include a legend in a spare panel (if available)?
addLegPanel = True #False

thisPlot = 1
thisPage = 1

# Loop over the budget variables
for nvar,var_label in enumerate(Var_labels_sub):
   #print(var_label)
   print(plot_titles[nvar])
   var_index=var_indices_sub[nvar]
   #print(var_index)

   #
   # check if need new page
   if thisPlot%maxPanels == 1:
      #
      # check if need to close old page first
      if thisPage > 1:
         fig.savefig(plotfile_pdf, bbox_inches="tight", pad_inches=0)
      #
      fig = plt.figure(facecolor='white', figsize=figSize)
      fig.set_tight_layout(True)
      plotfile = plotdir + '/Fig6_icebudget_ref_scycles_mainterms_' + plotTag + '_page{}.png'.format(thisPage)
      plotfile_pdf = plotdir + '/Fig6_icebudget_ref_scycles_mainterms_' + plotTag + '_page{}.pdf'.format(thisPage) 
     #
      # rest plot and advance page
      thisPlot = 1
      thisPage += 1
      #
   # Plot panel
   ax1 = plt.subplot(geomY, geomX, thisPlot) 

   for model in ModelNames:

     filepath=path+DataDic[model]['dir']
     Histfiles=DataDic[model]['HistFiles_ice']
     nhistfiles=len(Histfiles)

     # Ref cycles -----------------------------------------------------
     # Collect together all the relevant seasonal cycles (30*nensembles)  
     nensembles=nhistfiles
     nyears=30 # 30 years for each hist file
     massbudget_allcycles=np.empty([nensembles*nyears,12,nvars],dtype='float') 
     for nensemble,Histfile in enumerate(Histfiles):
       years,months,icearea,icemass,massbudget = rb.read_icefile(filepath+Histfile,nskip)	
       years_sub,icearea_sub,icemass_sub,massbudget_sub = bu.extract_years(years,
						icearea,icemass,massbudget,1960,1989)
       a=nensemble*nyears
       b=a+nyears
       massbudget_allcycles[a:b,:,:] = massbudget_sub[:,:,var_indices_sub]*seconds_in_month
       
     # Form the ref cycles of the mean and st dev
     massbudget_allcycles = massbudget_allcycles/1.0e15 # Convert from kg to Gt x10^3 
     massbudget_ref_mn=np.mean(massbudget_allcycles,axis=0)
     massbudget_ref_stdev=np.std(massbudget_allcycles,axis=0)

     mean=massbudget_ref_mn[:,nvar]
     stdev=massbudget_ref_stdev[:,nvar]
     if (np.min(mean)!= np.max(mean)): # Omit lines that are all zero
      ax1.plot(months,mean, 
       color=DataDic[model]['line_colour2'], 
       linestyle='-',
       #marker=BudgetLineDic['ice']['lineMarkers'][var_index],
       marker='.',
       label=DataDic[model]['label'],
       linewidth=1)
     #ax1.fill_between(months,mean-stdev,mean+stdev,
       #color=DataDic[model]['line_colour2'], alpha=0.25,
       #linestyle='-',
       #linewidth=0.5,
       #antialiased=True)

   ax1.axhline(0, color='black') # Add zero line
   # set x/y limits and titles
   ax1.set_title(plot_titles[nvar])
   ax1.set_ylabel(r'Mass flux (Gt/month x$10^3$)')
   #ax1.set_xlabel('Month')
   #
   thisPlot += 1

# if final page not full then add legend (if required) and close
if thisPlot%maxPanels != 1:
   #
   # add legend if required
   if addLegPanel:
      # create extra panel and remove axis
      ax1 = plt.subplot(geomY, geomX, thisPlot) 
      ax1.axis('off')
      #
      # create dummy lines for legend
      dummies=[]
      for model in ModelNames:
         thisLine = ax1.plot([], [], 
                        color=DataDic[model]['line_colour2'],
                        #edgecolor='grey',
                        marker='.',
                        label=DataDic[model]['label'], linestyle='-', linewidth=1)
         #
         dummies = dummies + thisLine
      #
      # set up a legend
      labs2 = [l.get_label() for l in dummies]
      axLeg = ax1.legend(dummies, labs2, loc='center left',
                           frameon=False,
 #                          borderpad=2,
                           prop={'size':10},
                           handleheight=1.0,
                           labelspacing=0.75)
   #
   # close final page
   fig.savefig(plotfile_pdf, bbox_inches="tight", pad_inches=0)

# close final page if needed
#if thisPlot%maxPanels != 1:
   fig.savefig(plotfile_pdf, bbox_inches="tight", pad_inches=0)

#XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


# ---------------That's all folks --------------------


